(function() {
	var preferences = self.options; 
	var callbacks = {};	
	
	var getNsPrefKey =  function(key) {
		return 'surfcanyon.' + key;
	}
	
	// [INTERNAL METHODS]
	
	var smallRandom = function() {
		return ('' + Math.random()).substring(2);
	};	
	
	// [MESSAGE HANDLER METHODS]
	
	// Listens for when setTimeout has expired, retrieves
	// the associated callback, then executes it
	self.port.on('timeoutComplete', function(payload) {
		if (payload.callbackKey && callbacks[payload.callbackKey]) {
			callbacks[payload.callbackKey]();
			delete callbacks[payload.callbackKey];
		}
	});	
		
	// Receives an AJAX response and a callback ID.
	// Retrieves the callback via ID and executes it,
	// passing the response as a parameter
	self.port.on("ajaxResponse", function(response) {
		if ((response.callbackKey) && callbacks[response.callbackKey]) {
			//Retreive the callback and execute
			callbacks[response.callbackKey](response);
			delete callbacks[response.callbackKey];
		}
	});	
	
	// Listens for successful changes to preferences, then mirrors
	// the preference state into a local variable
	self.port.on('prefchange', function(payload){
		preferences[payload.key] = payload.value;
	});

	// Listens for the receipt of sanitized HTML strings,
	// then retrieves, executes, and clears the appropriate
	// callback, in which the HTML content is added to the DOM
	self.port.on('sanitized', function(payload){
		callbacks[payload.callbackKey](payload.html);
		delete callbacks[payload.callbackKey];
	});	
		
	// [WINDOW METHODS]	
		
	// Jetpack addon's don't have access to setTimeout on
	// a page runtime. Call the background page and use the
	// named callback pattern.
	window.setFFTimeout = function(func, ms) {
		//Functions can't be passed to the background page.
		//Register the callback, retrieve by key, then call
		//so as not to lose its scope.
		var callbackKey = smallRandom();
		callbacks[callbackKey] = func;
		self.port.emit('setTimeout', {callbackKey: callbackKey, ms: ms});
	};
	
	openNewTab = function(url) {
		self.port.emit('openTab', {url: encodeURIComponent(url)});
	};
	
	getBoolPreferenceValue = function(key) {
		return preferences[getNsPrefKey(key)];
	};
	
	setBoolPreferenceValue = function(key, value) {
		self.port.emit('setPref', {key: getNsPrefKey(key), value: value});
	};

	getCharPreferenceValue = function(key) {
		return preferences[getNsPrefKey(key)];
	};

	setCharPreferenceValue = function(key, value) {
		self.port.emit('setPref', {key: getNsPrefKey(key), value: value.toString()});
	};
	
	getNode = function(rootNode, array) {
		for (var i=0; i<rootNode.childCount; i++) {
			 var node = rootNode.getChild(i);
			 if (node.type == 0) {
				 array.push(node);
			 } else if (node.type==6) {
				 this.getNode(node, array);
			 }
		}
	}	
	
	// Content script async call to the HTML sanitizer in main.js
	// Register the callback operation for later use, then request the
	// HTML content be sanitized.
	sc_appendSafeHtml = function(html, callback) {
		var callbackKey = smallRandom();
		callbacks[callbackKey] = callback;
		self.port.emit('sanitize', {callbackKey: callbackKey, html: html});
	};
	
	sc_alert = function(msg) {
		self.port.emit('prompt', {message: msg});
	};
	
	sc_ajax = function(params) {
		params.method = params.method||'GET';
		//in case method is defined, upper case it for
		//consumption by the background API
		params.method = params.method.toUpperCase();
		
		if (params.onerror) {
			var callbackKey = smallRandom();
			callbacks[callbackKey] = params.onerror;
			params.errKey = callbackKey;
		}
		
		//Functions can't be passed to the background page.
		//Register the callback, retrieve by key, then call
		//so as not to lose its scope.
		if (params.onload) {
			var callbackKey = smallRandom();
			callbacks[callbackKey] = params.onload;
			params.sKey = callbackKey;
		}
		self.port.emit(params.method, params);	
	}
})();
