var data = require("sdk/self").data;
var tabs = require('sdk/tabs');
var Requester = require("sdk/request");
var prefSet = require("sdk/simple-prefs");
var ss = require("sdk/simple-storage");
const {Cc} = require('chrome');
const {Ci} = require('chrome');
var parser = Cc["@mozilla.org/parserutils;1"].getService(Ci.nsIParserUtils);
var prompts = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
var { MatchPattern } = require("sdk/util/match-pattern");
var pattern = new MatchPattern(/.*(google|surfcanyon|localhost).*/);

let { setTimeout } = require('sdk/timers');

function openTab(url) {
	tabs.open(decodeURIComponent(url));
}

tabs.on('ready', function(tab) {
	try {
		var domain = tab.url.substr(0, tab.url.indexOf('/', tab.url.indexOf('//') + 2));
		if (!pattern.test(domain)) {
			return;
		} 
	} catch (e) {
		return;
	}
	
	var worker = tab.attach({	
		contentScriptOptions: prefSet.prefs,
		contentScriptFile: [data.url("sc_misc.js"), data.url("surfcanyon.js")]	  
	});
		
	function onPrefChange(prefName) {
		worker.port.emit('prefchange', {key: prefName, value: prefSet.prefs[prefName]});
	}
	
	// [MESSAGE HANDLER METHODS]
	
	// Receive messages from sc_misc.js and execute
	// the requested task
	
	// Handles AJAX GET requests
	worker.port.on("GET", function(params) {
		var req = new Requester.Request({
			url: params.url,
			onComplete: function (response) {
				var callbackKey = (response.status == 200) ? params.sKey : params.errKey||null;
				worker.port.emit('ajaxResponse', {responseText: response.text, callbackKey: callbackKey});
			}
		});
		
		req.get();
	});

	// Handles AJAX POST requests
	worker.port.on("POST", function(params) {
		var req = new Requester.Request({
			url: params.url,
			content: params.data,
			haders: {
				'Content-type' : 'application/x-www-form-urlencoded'
			},
			onComplete: function (response) {
				var callbackKey = (response.status == 200) ? params.sKey : params.errKey||null;
				worker.port.emit('ajaxResponse', {responseText: response.text, callbackKey: callbackKey});
			}
		});
		
		req.post();
	});
	
	// setTimeout is not directly available to content scripts.
	// This message listener emulates the behavior using the 'sdk/timers'
	// module. 
	worker.port.on('setTimeout', function(data) {
		setTimeout(function() {
			worker.port.emit('timeoutComplete', {callbackKey: data.callbackKey});
		}, data.ms);
	});
	
	worker.port.on('setPref', function(data) {
		prefSet.prefs[data.key] = data.value;
		onPrefChange(data.key);
	});
	
	worker.port.on('openTab', function(message) {
		openTab(decodeURIComponent(message.url));
	});	
	
	// Sanitizes HTML input and passes the safe HTML
	// string back to the content script by way of message
	// passing.
	worker.port.on('sanitize', function(message) {
		message.html = parser.sanitize(message.html, 2);
		worker.port.emit('sanitized', message);
	});	
	
	worker.port.on('prompt', function(message) {
		prompts.alert(null, "Surf Canyon", message.message);
	});
});

// show installed splash screen on installation
exports.main = function(options, callbacks) {
	if (options.loadReason === 'install') {
		openTab('http://www.surfcanyon.com/installed.jsp');
	} 
};

// Inert until bug 627432 is fixed - on uninstall the correct unload
// reason is not currently being sent.
// Bug report here: https://bugzilla.mozilla.org/show_bug.cgi?id=627432
exports.onUnload = function(reason) {
	if (reason === 'uninstall') {
		openTab('http://www.surfcanyon.com/uninstall.jsp');
	}
};
	
